/* 
   SunflowerApplet, A GNOME applet for quickly changing screen geometry.

   Copyright 2003 Matthew Allum <mallum@openedhand.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include <panel-applet-gconf.h>
#include <panel-applet.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <X11/Xproto.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>
#include <X11/extensions/Xrender.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf-value.h>
#include <time.h>
#include <math.h>

#include "config.h"

#define SHADOW_OFFSET_X		"/apps/compiz/plugins/decoration/allscreens/options/shadow_offset_x"
#define SHADOW_OFFSET_Y		"/apps/compiz/plugins/decoration/allscreens/options/shadow_offset_y"
#define SHADOW_OPACITY		"/apps/compiz/plugins/decoration/allscreens/options/shadow_opacity"
#define SHADOW_RADIUS		"/apps/compiz/plugins/decoration/allscreens/options/shadow_radius"

typedef struct
{
  GtkWidget *applet;
  GtkWidget *frame;
  GtkWidget *image;
  GtkWidget *latit;
  GtkWidget *longi;
  GtkWidget *lat_deg;
  GtkWidget *lat_min;
  GtkWidget *long_deg;
  GtkWidget *long_min;
  GtkWidget *sunrise_label;
  GtkWidget *sunset_label;
  GKeyFile *emerald_theme;

  gint size;
  GConfClient *gconf;
  char filename[1024];
  gint sunrise;
  gint sunset;
  gint lasthour;
  GdkPixbuf *pixbuf;
  GdkPixbuf *map;
  gfloat latitude;
  gfloat longitude;
  gfloat tzshift;

} SunflowerData;

static void
sunflower_dialog_about (BonoboUIComponent * uic,
			SunflowerData * sunflower, const gchar * verbname);
static void
sunflower_dialog_prefs (BonoboUIComponent * uic,
			SunflowerData * sunflower, const gchar * verbname);

gint timeout_cb (SunflowerData * sunflower);
gfloat calcsunrise (gint month, gint day, gfloat latitude, gfloat longitude);
gfloat calcsunset (gint month, gint day, gfloat latitude, gfloat longitude);
gfloat declination (gint month, gint day);
gfloat fnacs (gfloat x);
gint calcmins (gfloat timeofday, gfloat tzshift);


static const char Context_menu_xml[] =
  "<popup name=\"button3\">\n"
  "   <menuitem name=\"Preferences Item\" "
  "             verb=\"VerbPreferences\" "
  "           _label=\"Preferences\"\n"
  "          pixtype=\"stock\" "
  "          pixname=\"gtk-properties\"/>\n"
  "   <menuitem name=\"About Item\" "
  "             verb=\"VerbAbout\" "
  "           _label=\"About ...\"\n"
  "          pixtype=\"stock\" "
  "          pixname=\"gnome-stock-about\"/>\n" "</popup>\n";

static const BonoboUIVerb Context_menu_verbs[] = {
  BONOBO_UI_UNSAFE_VERB ("VerbAbout", sunflower_dialog_about),
  BONOBO_UI_UNSAFE_VERB ("VerbPreferences", sunflower_dialog_prefs),
  BONOBO_UI_VERB_END
};

BonoboGenericFactory *Applet_pointer;	// Pointer to the Applet

static void
sunflower_dialog_about (BonoboUIComponent * uic,
			SunflowerData * sunflower, const gchar * verbname)
{
  static GtkWidget *about = NULL;
  gchar *file;

  static const gchar *authors[] = {
    "Kevin DeKorte <kdekorte@yahoo.com>",
    NULL
  };

  gtk_show_about_dialog(NULL, "name","Sunflower",
                        "logo", sunflower->pixbuf,
                        "authors", authors,
                        "copyright", "Copyright © 2007 Kevin DeKorte",
                        "comments", "Changes the shadow on the windows in compiz with the movement of the sun",
                        "version", VERSION, NULL);
}

gint
close_prefs (GtkWidget * widget, GtkWidget * prefs)
{

  if (GTK_IS_WIDGET (prefs))
    gtk_widget_destroy (prefs);
  prefs = NULL;

  return (FALSE);

}

gint
pos_adjust (GtkWidget * widget, SunflowerData * sunflower)
{
  gfloat deg;
  int hour;
  gchar *text;
  struct tm *lt;
  time_t t;

  sunflower->latitude =
    gtk_spin_button_get_value (GTK_SPIN_BUTTON (sunflower->lat_min)) / 60.0;
  deg = gtk_spin_button_get_value (GTK_SPIN_BUTTON (sunflower->lat_deg));
  if (deg < 0.0)
    {
      sunflower->latitude = deg - sunflower->latitude;

    }
  else
    {
      sunflower->latitude = deg + sunflower->latitude;
    }

  sunflower->longitude =
    gtk_spin_button_get_value (GTK_SPIN_BUTTON (sunflower->long_min)) / 60.0;
  deg = gtk_spin_button_get_value (GTK_SPIN_BUTTON (sunflower->long_deg));
  if (deg < 0.0)
    {
      sunflower->longitude = deg - sunflower->longitude;

    }
  else
    {
      sunflower->longitude = deg + sunflower->longitude;
    }

  panel_applet_gconf_set_float ((PanelApplet *)sunflower->applet, "latitude",
				sunflower->latitude, NULL);
  panel_applet_gconf_set_float ((PanelApplet *)sunflower->applet, "longitude",
				sunflower->longitude, NULL);


  t = time (NULL);
  lt = localtime (&t);

  sunflower->sunrise = calcmins (calcsunrise (lt->tm_mon + 1,
					      lt->tm_mday,
					      sunflower->latitude,
					      sunflower->longitude),
				 sunflower->tzshift);

  sunflower->sunset = calcmins (calcsunset (lt->tm_mon + 1,
					    lt->tm_mday,
					    sunflower->latitude,
					    sunflower->longitude),
				sunflower->tzshift);


  hour = sunflower->sunrise / 60;
  text = g_strdup_printf ("%i:%02i", hour, sunflower->sunrise - (hour * 60));
  gtk_label_set_text (GTK_LABEL (sunflower->sunrise_label), text);
  g_free (text);

  hour = sunflower->sunset / 60;
  text = g_strdup_printf ("%i:%02i", hour, sunflower->sunset - (hour * 60));
  gtk_label_set_text (GTK_LABEL (sunflower->sunset_label), text);
  g_free (text);
  
  return (FALSE);

}

gboolean select_callback(GtkWidget * widget, GdkEventButton * event, SunflowerData * sunflower)
{
    
    if (event->button == 1) {
        sunflower->latitude = (event->y - 90) / -1;
        gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->lat_deg),
                                   (gint)(sunflower->latitude));
        sunflower->longitude = event->x - 180;
        gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->long_deg),
                                   (gint)(sunflower->longitude));
    
        gdk_draw_pixbuf(widget->window,NULL,sunflower->map,0,0,0,0,-1,-1,GDK_RGB_DITHER_NONE,0,0);
        gdk_draw_arc(widget->window,widget->style->fg_gc[GTK_STATE_NORMAL],TRUE,event->x-2,event->y-2,5,5,0,360*64);
    
    }
    return FALSE;
}

gboolean image_expose(GtkWidget * widget, GdkEventExpose * event, SunflowerData * sunflower)
{
    gint x,y;

    x = sunflower->longitude + 180;
    y = sunflower->latitude * -1 + 90;
    gdk_draw_pixbuf(widget->window,NULL,sunflower->map,0,0,0,0,-1,-1,GDK_RGB_DITHER_NONE,0,0);
    gdk_draw_arc(widget->window,widget->style->fg_gc[GTK_STATE_NORMAL],TRUE,x-2,y-2,5,5,0,360*64);

    return TRUE; 
}

static void
sunflower_dialog_prefs (BonoboUIComponent * uic,
			SunflowerData * sunflower, const gchar * verbname)
{
  static GtkWidget *prefs = NULL;
  GtkWidget *vbox;
  GtkWidget *hbutton_box;
  GtkWidget *cancel;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *select_map;
  GtkWidget *image_map;
  gchar *text;
  gint hour;


  prefs = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_screen (GTK_WINDOW (prefs),
			 gtk_widget_get_screen (sunflower->applet));


  vbox = gtk_vbox_new (FALSE, 10);
  hbutton_box = gtk_hbutton_box_new ();
  table = gtk_table_new (20, 3, FALSE);

  label = gtk_label_new ("");
  gtk_label_set_markup (GTK_LABEL (label), "<b>Location</b>");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  gtk_container_add (GTK_CONTAINER (vbox), label);
  
  select_map = gtk_event_box_new();
  image_map = gtk_image_new_from_pixbuf(sunflower->map);
  gtk_widget_set_events(select_map, GDK_BUTTON_PRESS_MASK);
  gtk_container_add(GTK_CONTAINER(select_map), image_map);
  gtk_container_add (GTK_CONTAINER (vbox), select_map);
  g_signal_connect(G_OBJECT(select_map),
                   "button_press_event", G_CALLBACK(select_callback), sunflower);
  g_signal_connect(G_OBJECT(image_map),
                   "expose_event", G_CALLBACK(image_expose), sunflower);
  
  gtk_container_add (GTK_CONTAINER (vbox), table);
//  gtk_container_add (GTK_CONTAINER (vbox), hbutton_box);
  gtk_container_add (GTK_CONTAINER (prefs), vbox);
  gtk_window_set_title (GTK_WINDOW (prefs), "Sunflower Preferences");
  gtk_container_set_border_width (GTK_CONTAINER (prefs), 5);

  cancel = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
  g_signal_connect (GTK_OBJECT (cancel), "clicked",
		    GTK_SIGNAL_FUNC (close_prefs), prefs);

  gtk_container_add (GTK_CONTAINER (hbutton_box), cancel);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbutton_box), GTK_BUTTONBOX_END);

  label = gtk_label_new ("Latitude:");
  sunflower->lat_deg = gtk_spin_button_new_with_range (-90, 90, 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->lat_deg),
			     (gint)(sunflower->latitude));
  sunflower->lat_min = gtk_spin_button_new_with_range (0, 99, 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->lat_min),
			     (fabs (sunflower->latitude) -
			      (gint)(fabs (sunflower->latitude))) * 60.0);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 12, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_table_attach(GTK_TABLE (table), sunflower->lat_deg,
                             1, 2, 0, 1, GTK_SHRINK,GTK_SHRINK,0,0);
  label = gtk_label_new ("°");
  gtk_table_attach(GTK_TABLE (table), label, 2, 3, 0, 1, GTK_SHRINK,GTK_SHRINK,0,0);
  gtk_table_attach(GTK_TABLE (table), sunflower->lat_min,
                             3, 4, 0, 1, GTK_SHRINK,GTK_SHRINK,0,0);
  label = gtk_label_new ("'");
  gtk_table_attach(GTK_TABLE (table), label, 4, 5, 0, 1, GTK_SHRINK,GTK_SHRINK,0,0);

  label = gtk_label_new ("Longitude:");
  sunflower->long_deg = gtk_spin_button_new_with_range (-180, 180, 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->long_deg),
			     (gint)(sunflower->longitude));
  sunflower->long_min = gtk_spin_button_new_with_range (0, 99, 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (sunflower->long_min),
			     (fabs (sunflower->longitude) -
			      (gint)(fabs (sunflower->longitude))) * 60.0);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 12, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
  gtk_table_attach(GTK_TABLE (table), sunflower->long_deg,
                             1, 2, 1, 2, GTK_SHRINK,GTK_SHRINK,0,0);
  label = gtk_label_new ("°");
  gtk_table_attach(GTK_TABLE (table), label, 2, 3, 1, 2, GTK_SHRINK,GTK_SHRINK,0,0);
  gtk_table_attach(GTK_TABLE (table), sunflower->long_min,
                   3, 4, 1, 2, GTK_SHRINK,GTK_SHRINK,0,0);
  label = gtk_label_new ("'");
  gtk_table_attach(GTK_TABLE (table), label, 4, 5, 1, 2, GTK_SHRINK,GTK_SHRINK,0,0);

  g_signal_connect (GTK_OBJECT (sunflower->lat_deg), "value-changed",
		    GTK_SIGNAL_FUNC (pos_adjust), sunflower);
  g_signal_connect (GTK_OBJECT (sunflower->lat_min), "value-changed",
		    GTK_SIGNAL_FUNC (pos_adjust), sunflower);
  g_signal_connect (GTK_OBJECT (sunflower->long_deg), "value-changed",
		    GTK_SIGNAL_FUNC (pos_adjust), sunflower);
  g_signal_connect (GTK_OBJECT (sunflower->long_min), "value-changed",
		    GTK_SIGNAL_FUNC (pos_adjust), sunflower);
  
  table = gtk_table_new (20, 3, FALSE);
  gtk_container_add (GTK_CONTAINER (vbox), table);
  gtk_container_add (GTK_CONTAINER (vbox), hbutton_box);

  label = gtk_label_new ("");
  gtk_label_set_markup (GTK_LABEL (label), "<b>Approximate Times:</b>");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);

  label = gtk_label_new ("Sunrise:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 12, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);

  hour = sunflower->sunrise / 60;
  text = g_strdup_printf ("%i:%02i", hour, sunflower->sunrise - (hour * 60));
  sunflower->sunrise_label = gtk_label_new (text);
  g_free (text);
  gtk_misc_set_alignment (GTK_MISC (sunflower->sunrise_label), 1.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table),
			     sunflower->sunrise_label, 1, 2, 3, 4);

  label = gtk_label_new ("Sunset:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 12, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 4, 5);

  hour = sunflower->sunset / 60;
  text = g_strdup_printf ("%i:%02i", hour, sunflower->sunset - (hour * 60));
  sunflower->sunset_label = gtk_label_new (text);
  g_free (text);
  gtk_misc_set_alignment (GTK_MISC (sunflower->sunset_label), 1.0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), sunflower->sunset_label,
			     1, 2, 4, 5);



  g_signal_connect (G_OBJECT (prefs), "destroy",
		    (GCallback) gtk_widget_destroyed, &prefs);
  gtk_window_set_position (GTK_WINDOW (prefs), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (prefs), FALSE);
  gtk_widget_show_all (prefs);

}

static void
applet_change_size_cb (PanelApplet * applet,
		       gint size, SunflowerData * sunflower)
{
  if (sunflower->size != size)
    {

      GdkPixbuf *scaled;

      sunflower->size = size;

      if (sunflower->pixbuf)
	{
	  scaled = gdk_pixbuf_scale_simple (sunflower->
					    pixbuf, size,
					    size, GDK_INTERP_BILINEAR);

	  gtk_image_set_from_pixbuf (GTK_IMAGE (sunflower->image), scaled);
	  gtk_widget_show (sunflower->image);

	}
    }
}


static void
sunflower_applet_new (PanelApplet * applet)
{
  SunflowerData *sunflower;
  struct tm *lt;
  time_t t;
  FILE *file;
  gchar *data = NULL;
  gboolean loaded;

  panel_applet_set_background_widget(applet, GTK_WIDGET(applet));
  sunflower = g_new0 (SunflowerData, 1);
  t = time (NULL);
  lt = localtime (&t);
  sunflower->tzshift = timezone / 3600;	// + daylight;

  /* Defaults 40.14,-105.17  */
  sunflower->size = 0;
  sunflower->latitude =
    panel_applet_gconf_get_float (applet, "latitude", NULL);
  sunflower->longitude =
    panel_applet_gconf_get_float (applet, "longitude", NULL);
  sunflower->lasthour = 24;

  /* Applet Widgets  */
  sunflower->frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (sunflower->frame), GTK_SHADOW_NONE);
  gtk_container_add (GTK_CONTAINER (applet), sunflower->frame);
  sunflower->pixbuf =
          gdk_pixbuf_new_from_file (DATADIR "pixmaps/sunflower.png", NULL);
  sunflower->map =
          gdk_pixbuf_new_from_file (DATADIR "pixmaps/worldmap.png", NULL);

  sunflower->image = gtk_image_new ();
  gtk_container_add (GTK_CONTAINER (sunflower->frame), sunflower->image);

  sunflower->applet = GTK_WIDGET (applet);
  sunflower->gconf = gconf_client_get_default ();

  // load emerald settings
  // file = fopen("/tmp/sunflower.log","w");
  sunflower->emerald_theme = NULL;
  snprintf (sunflower->filename, 1024, "%s/.emerald/theme/theme.ini",
	    getenv ("HOME"));
  // fprintf(file,"opening %s\n",sunflower->filename);
  if (g_file_test(sunflower->filename, G_FILE_TEST_EXISTS)) {
    sunflower->emerald_theme = g_key_file_new ();
    loaded = g_key_file_load_from_file (sunflower->emerald_theme,
				      sunflower->filename,
				      G_KEY_FILE_KEEP_COMMENTS |
				      G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
  }

  /* Set default settings */
  gconf_client_set_float (sunflower->gconf, SHADOW_RADIUS, 8.0, NULL);
  gconf_client_set_float (sunflower->gconf, SHADOW_OPACITY, 0.5, NULL);


  if (sunflower->emerald_theme) {
    g_key_file_set_integer (sunflower->emerald_theme, "shadow", "shadow_radius",
			  8);
    g_key_file_set_double (sunflower->emerald_theme, "shadow", "shadow_opacity",
			 0.5);
    file = fopen (sunflower->filename, "w");
    data = g_key_file_to_data (sunflower->emerald_theme, NULL, NULL);
    fprintf (file, "%s", data);
    fclose (file);
    g_free (data);
    g_key_file_free (sunflower->emerald_theme);
  }
  // hook up callback and initialize
  timeout_cb (sunflower);
  g_timeout_add (60 * 1000, (GtkFunction)timeout_cb, sunflower);
  // Use this line for testing, helps find bugs quicker
  // gtk_timeout_add (100, timeout_cb, sunflower);

  g_signal_connect (sunflower->applet, "change_size",
		    G_CALLBACK (applet_change_size_cb), sunflower);

  panel_applet_setup_menu (PANEL_APPLET (applet),
			   Context_menu_xml, Context_menu_verbs, sunflower);

  /* Make sure the panel image gets painted */
  applet_change_size_cb (applet, panel_applet_get_size (applet), sunflower);

  gtk_widget_show_all (GTK_WIDGET (applet));
  // fprintf(file,"%s",g_key_file_to_data(sunflower->emerald_theme,NULL,NULL));
  // fclose(file);
}


gint
timeout_cb (SunflowerData * sunflower)
{
  struct tm *lt;
  time_t t;
  float angle, rangle;
  int x;
  int y;
  int mins;
  FILE *file;
  gchar *data = NULL;

  t = time (NULL);
  lt = localtime (&t);

  mins = lt->tm_hour * 60 + lt->tm_min;

  if (sunflower->lasthour > lt->tm_hour)
    {
      sunflower->sunrise = calcmins (calcsunrise (lt->tm_mon + 1,
						  lt->tm_mday,
						  sunflower->
						  latitude,
						  sunflower->
						  longitude),
				     sunflower->tzshift);

      sunflower->sunset = calcmins (calcsunset (lt->tm_mon + 1,
						lt->tm_mday,
						sunflower->
						latitude,
						sunflower->
						longitude),
				    sunflower->tzshift);

    }
  sunflower->lasthour = lt->tm_hour;
  angle = (((float) mins - (float) sunflower->sunrise) /
	   ((float) sunflower->sunset - (float) sunflower->sunrise)) * 180;
  rangle = angle * (3.14 / 180);

  x = -(int) (cos (rangle) * 16);
  y = (int) (sin (rangle) * 16);

  sunflower->emerald_theme = NULL;

  if (g_file_test(sunflower->filename, G_FILE_TEST_EXISTS)) {
    sunflower->emerald_theme = g_key_file_new ();
    g_key_file_load_from_file (sunflower->emerald_theme,
			     sunflower->filename,
			     G_KEY_FILE_KEEP_COMMENTS |
			     G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
  }

  if (y >= 0)
    {
      gconf_client_set_int (sunflower->gconf, SHADOW_OFFSET_X, x, NULL);
      gconf_client_set_int (sunflower->gconf, SHADOW_OFFSET_Y, y, NULL);
      
      if (sunflower->emerald_theme) {
      g_key_file_set_integer (sunflower->emerald_theme, "shadow",
			      "shadow_offset_x", x);
      g_key_file_set_integer (sunflower->emerald_theme, "shadow",
			      "shadow_offset_y", y);
      }
    }
  else
    {
      gconf_client_set_int (sunflower->gconf, SHADOW_OFFSET_X, 5, NULL);
      gconf_client_set_int (sunflower->gconf, SHADOW_OFFSET_Y, 5, NULL);
      if (sunflower->emerald_theme) {
      g_key_file_set_integer (sunflower->emerald_theme, "shadow",
			      "shadow_offset_x", 5);
      g_key_file_set_integer (sunflower->emerald_theme, "shadow",
			      "shadow_offset_y", 5);
        }
    }

  if (sunflower->emerald_theme)
    {
      data = g_key_file_to_data (sunflower->emerald_theme, NULL, NULL);
      file = fopen (sunflower->filename, "w");
      fprintf (file, "%s", data);
      fclose (file);
      g_free (data);
      g_key_file_free (sunflower->emerald_theme);
      g_spawn_command_line_async ("killall -USR1 emerald", NULL);
    }
  return 1;
}


static gboolean
sunflower_applet_factory (PanelApplet * applet, const char *id, gpointer data)
{

  sunflower_applet_new (applet);
  return TRUE;

}


gfloat
declination (gint month, gint day)
{
  // Approximation for sun declination
  gfloat dec;
  gfloat pi = 3.1415926;

  dec = day + 30.3 * (month - 1);
  dec = 10 + .988 * dec;
  dec = -23.5 * cos (dec * pi / 180);
  return dec;
}

gfloat
fnacs (gfloat x)
{
  gfloat pi = 3.1415926;
  return pi / 2 - atan (x / sqrt (1 - x * x));
}

gfloat
calcsunrise (gint month, gint day, gfloat latitude, gfloat longitude)
{

  gfloat dec;
  gfloat x, y, z;
  gfloat pi = 3.1415926;

  dec = declination (month, day);
  x = longitude / 15;
  y = tan (dec * pi / 180) * tan (latitude * pi / 180);
  if (y > 1 || y < -1)
    {
      return -1;
    }
  else
    {
      z = fnacs (y) * 180 / pi / 15;
      return x + z;
    }

}

gfloat
calcsunset (gint month, gint day, gfloat latitude, gfloat longitude)
{
  gfloat dec;
  gfloat x, y, z;
  gfloat pi = 3.1415926;

  dec = declination (month, day);
  x = longitude / 15;
  y = tan (dec * pi / 180) * tan (latitude * pi / 180);
  if (y > 1 || y < -1)
    {
      return -1;
    }
  else
    {
      z = fnacs (y) * 180 / pi / 15;
      return x - z;
    }

}

gint
calcmins (gfloat timeofday, gfloat tzshift)
{

  gint hour, min;

  if (timeofday < 0)
    {
      timeofday += 24;
    }
  if (timeofday > 24)
    {
      timeofday -= 24;
    }
  hour = floor (timeofday);
  min = (timeofday - hour) * 60;
  if (min == 60)
    {
      min = 0;
      hour++;
    }
  hour += tzshift;
  if (hour < 0)
    {
      hour += 24;
    };
  if (hour > 24)
    {
      hour -= 24;
    };
  return hour * 60 + min;
}




PANEL_APPLET_BONOBO_FACTORY ("OAFIID:SunflowerApplet_factory",
			     PANEL_TYPE_APPLET,
			     "sunflower",
			     "0", sunflower_applet_factory, NULL);
